package fr.openium.consentium.data.di

import android.content.Context
import dagger.Lazy
import dagger.Module
import dagger.Provides
import dagger.Reusable
import dagger.hilt.InstallIn
import dagger.hilt.components.SingletonComponent
import fr.openium.consentium.data.remote.ConsentiumApi
import kotlinx.serialization.json.Json
import okhttp3.Cache
import okhttp3.HttpUrl
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.OkHttpClient
import retrofit2.Retrofit
import retrofit2.converter.kotlinx.serialization.asConverterFactory
import java.util.concurrent.TimeUnit
import javax.inject.Qualifier
import javax.inject.Singleton

private val json by lazy {
    Json {
        isLenient = true
        ignoreUnknownKeys = true
    }
}

@Module
@InstallIn(SingletonComponent::class)
internal object NetworkModule {

    @OkHttpClientDefault
    @Provides
    @Singleton
    fun okHttpClientDefault(
        okHttpBuilder: OkHttpClient.Builder,
    ): OkHttpClient = okHttpBuilder.build()

    fun standardOkHttpBuilder(
        context: Context,
    ): OkHttpClient.Builder {
        val cacheSize = (20 * 1024 * 1024).toLong() // 20 MiB
        val cache = Cache(context.cacheDir, cacheSize)
        return OkHttpClient.Builder()
            .cache(cache)
            .connectTimeout(10, TimeUnit.SECONDS)
            .readTimeout(10, TimeUnit.SECONDS)
            .writeTimeout(10, TimeUnit.SECONDS)
    }

    @Reusable
    @Provides
    fun provideConsentiumApi(
        @ConsentiumUrl url: HttpUrl,
        @OkHttpClientDefault okHttpClient: Lazy<OkHttpClient>,
    ): ConsentiumApi =
        createRetrofit(
            url,
            okHttpClient
        ).create(ConsentiumApi::class.java)

    private fun createRetrofit(url: HttpUrl, okHttpClient: Lazy<OkHttpClient>): Retrofit =
        Retrofit.Builder()
            .callFactory { request ->
                okHttpClient.get().newCall(request)
            }.addConverterFactory(json.asConverterFactory("application/json".toMediaType()))
            .baseUrl(url)
            .build()
}

@Qualifier
@Retention(AnnotationRetention.BINARY)
internal annotation class ConsentiumUrl

@Qualifier
@Retention(AnnotationRetention.BINARY)
internal annotation class OkHttpClientDefault
